/// <reference types="react" />
import { AnyT, StringNumberT } from '../../../definitions.interface';
import { DateFieldRangeValueT, DateTimeLikeTypeT, IBaseEvent, IDatesRangeEntity } from '../../../definition';
import { IDateFieldProps, IDateFieldState } from './date-field.interface';
import { BaseTextField } from '../text-field';
export declare class DateField<TProps extends IDateFieldProps = IDateFieldProps, TState extends IDateFieldState = IDateFieldState> extends BaseTextField<TProps, TState> {
    static readonly defaultProps: IDateFieldProps;
    private readonly defaultRangeFieldProps;
    private readonly dialogRef;
    private readonly fromDateRef;
    /**
     * @stable [07.01.2020]
     * @param {TProps} props
     */
    constructor(props: TProps);
    /**
     * @stable [07.03.2020]
     * @param {IDatesRangeEntity | DateTimeLikeTypeT | AnyT} currentRawValue
     */
    onChangeManually(currentRawValue: IDatesRangeEntity | DateTimeLikeTypeT | AnyT): void;
    /**
     * @stable [09.01.2020]
     * @returns {JSX.Element}
     */
    protected getAttachmentElement(): JSX.Element;
    /**
     * @stable [07.03.2020]
     * @returns {StringNumberT[] | string}
     */
    protected readonly originalEmptyValue: StringNumberT[] | string;
    /**
     * @stable [07.03.2020]
     * @param {IBaseEvent} event
     */
    protected onClick(event: IBaseEvent): void;
    /**
     * @stable [07.03.2020]
     * @param {DateFieldRangeValueT} value
     * @returns {string}
     */
    protected decorateDisplayValue(value: DateFieldRangeValueT): string;
    /**
     * @stable [09.03.2020]
     * @returns {string}
     */
    protected getFieldClassName(): string;
    /**
     * @stable [09.03.2020]
     * @returns {string}
     */
    protected getFieldPattern(): string;
    /**
     * @stable [09.03.2020]
     * @returns {Array<string | RegExp>}
     */
    protected getFieldMask(): Array<string | RegExp>;
    /**
     * @stable [09.03.2020]
     * @returns {string}
     */
    protected readonly fieldFormat: string;
    /**
     * @stable [22.01.2020]
     * @returns {JSX.Element}
     */
    private readonly calendarAttachmentElement;
    /**
     * @stable [20.01.2020]
     * @returns {JSX.Element}
     */
    private readonly calendarElement;
    /**
     * @stable [21.01.2020]
     * @returns {ICalendarProps}
     */
    private readonly defaultCalendarProps;
    /**
     * @stable [20.01.2020]
     * @returns {React.ReactNode}
     */
    private readonly rangeExplorerDateElement;
    /**
     * @stable [20.01.2020]
     * @returns {JSX.Element}
     */
    private readonly dateFieldsElement;
    /**
     * @stable [07.03.2020]
     * @returns {JSX.Element}
     */
    private readonly quickButtonsElement;
    /**
     * @stable [07.03.2020]
     */
    private onSetDay;
    /**
     * @stable [07.03.2020]
     */
    private onSetWeek;
    /**
     * @stable [07.03.2020]
     */
    private onSetMonth;
    /**
     * @stable [08.03.2020]
     */
    private onSetQuarter;
    /**
     * @stable [08.03.2020]
     */
    private onSetYear;
    /**
     * @stable [08.03.2020]
     */
    private onSetCustom;
    /**
     * @stable [06.03.2020]
     * @param {Date} from
     * @param {Date} to
     * @param {DatePeriodsEnum} periodMode
     */
    private setQuickValue;
    /**
     * @stable [11.03.2020]
     */
    private setFromDateFocus;
    /**
     * @stable [07.03.2020]
     */
    private setPreviousMonth;
    /**
     * @stable [07.03.2020]
     */
    private setNextMonth;
    /**
     * @stable [07.03.2020]
     * @param {number} duration
     */
    private doChangeMonth;
    /**
     * @stable [09.03.2020]
     * @param {string} value
     */
    private onToDateFieldChange;
    /**
     * @stable [11.03.2020]
     * @param {string} value
     */
    private onFromDateFieldChange;
    /**
     * @stable [10.03.2020]
     */
    private onAccept;
    /**
     * @stable [09.03.2020]
     * @param {ICalendarDayEntity} calendarDayEntity
     */
    private onDaySelect;
    /**
     * @stable [08.03.2020]
     */
    private onDialogDeactivate;
    /**
     * @stable [08.03.2020]
     * @param {() => void} callback
     */
    private onDialogClose;
    /**
     * @stable [08.03.2020]
     */
    private openDialog;
    /**
     * @stable [09.03.2020]
     * @param {IDatesRangeEntity} dateRangeEntity
     * @returns {string}
     */
    private getDecoratedRangeDisplayValue;
    /**
     * @stable [09.03.2020]
     * @param {DateTimeLikeTypeT} from
     * @param {DateTimeLikeTypeT} to
     * @param {string} label
     * @returns {string}
     */
    private getDecoratedRangeSpecificValue;
    /**
     * @stable [09.03.2020]
     * @param {string} dateFromAsString
     * @param {string} dateToAsString
     * @param {Date} dateFrom
     * @param {Date} dateTo
     * @returns {boolean}
     */
    private checkRange;
    /**
     * @stable [08.03.2020]
     * @param {DateTimeLikeTypeT} value
     * @returns {string}
     */
    private serializeValue;
    /**
     * @stable [08.03.2020]
     * @returns {string}
     */
    private readonly currentDateAsDisplayValue;
    /**
     * @stable [08.03.2020]
     * @returns {string}
     */
    private readonly nextDateAsDisplayValue;
    /**
     * @stable [08.03.2020]
     * @returns {ICalendarEntity}
     */
    private readonly currentCalendarEntity;
    /**
     * @stable [08.03.2020]
     * @returns {ICalendarEntity}
     */
    private readonly nextCalendarEntity;
    /**
     * @stable [08.03.2020]
     * @param {Date} date
     * @returns {ICalendarEntity}
     */
    private getCalendarEntity;
    /**
     * @stable [07.03.2020]
     * @returns {Date}
     */
    private readonly currentCalendarDate;
    /**
     * @stable [08.03.2020]
     * @returns {Date}
     */
    private readonly nextCalendarDate;
    /**
     * @stable [07.03.2020]
     * @returns {IFromToDayOfYearEntity}
     */
    private readonly currentFromToDayOfYearEntity;
    /**
     * @stable [07.03.2020]
     * @param {IDatesRangeEntity} entity
     * @returns {DatesRangeValueT}
     */
    private fromDatesRangeEntity;
    /**
     * @stable [07.03.2020]
     * @param {DatesRangeValueT} value
     * @returns {IDatesRangeEntity}
     */
    private fromDatesRangeValue;
    /**
     * @stable [07.03.2020]
     * @returns {IDatesRangeEntity}
     */
    private readonly datesRangeEntity;
    /**
     * @stable [06.03.2020]
     * @param {DateTimeLikeTypeT} date
     * @returns {Date}
     */
    private getValueAsDate;
    /**
     * @stable [10.03.2020]
     * @returns {IDatesRangeEntity}
     */
    private readonly acceptedDatesRangeEntity;
    /**
     * @stable [06.03.2020]
     * @returns {Date}
     */
    private readonly acceptedDateValue;
    /**
     * @stable [06.03.2020]
     * @returns {Date}
     */
    private readonly valueAsDate;
    /**
     * @stable [06.03.2020]
     * @returns {DatePeriodsEnum}
     */
    private readonly valueAsPeriodMode;
    /**
     * @stable [08.03.2020]
     * @returns {DatePeriodsEnum}
     */
    private readonly valueAsDateFrom;
    /**
     * @stable [09.03.2020]
     * @returns {DateTimeLikeTypeT}
     */
    private readonly valueAsDateTo;
    /**
     * @stable [06.03.2020]
     * @returns {DateFieldRangeValueT}
     */
    private readonly aheadValue;
    /**
     * @stable [07.03.2020]
     * @returns {IDatesRangeEntity}
     */
    private readonly aheadEntity;
    /**
     * @stable [07.03.2020]
     * @returns {DatePeriodsEnum}
     */
    private readonly aheadEntityPeriodMode;
    /**
     * @stable [07.03.2020]
     * @returns {DatePeriodsEnum}
     */
    private readonly selectedPeriodMode;
    /**
     * @stable [07.03.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    private isDaySelected;
    /**
     * @stable [08.03.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    private isRangeSelected;
    /**
     * @stable [07.03.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    private isFirstDaySelected;
    /**
     * @stable [07.03.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    private isLastDaySelected;
    /**
     * @stable [07.03.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    private isMiddleDaySelected;
    /**
     * @stable [07.03.2020]
     * @param {IFromToDayOfYearEntity} currentFromToDayOfYearEntity
     * @returns {boolean}
     */
    private isServiceDaySelected;
    /**
     * @stable [06.03.2020]
     * @returns {boolean}
     */
    private readonly isPreviousPeriodModeEnabled;
    /**
     * @stable [06.03.2020]
     * @returns {boolean}
     */
    private readonly isRangeEnabled;
    /**
     * @stable [06.03.2020]
     * @returns {boolean}
     */
    private readonly isInline;
    /**
     * @stable [09.03.2020]
     * @returns {boolean}
     */
    private readonly isFieldMaskNeeded;
    /**
     * stable [06.03.2020]
     * @returns {IDateTimeSettingsEntity}
     */
    private readonly dateTimeSettings;
    /**
     * @stable [06.03.2020]
     * @returns {Dialog}
     */
    private readonly dialog;
}
