"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var util_1 = require("../../../util");
var definitions_interface_1 = require("../../../definitions.interface");
var definition_1 = require("../../../definition");
var text_field_1 = require("../text-field");
var button_1 = require("../../button");
var calendar_1 = require("../../calendar");
var dialog_1 = require("../../dialog");
var DateField = /** @class */ (function (_super) {
    __extends(DateField, _super);
    /**
     * @stable [07.01.2020]
     * @param {TProps} props
     */
    function DateField(props) {
        var _this = _super.call(this, props) || this;
        _this.defaultRangeFieldProps = {
            calendarActionRendered: false,
            errorMessageRendered: false,
            full: false,
            placeholder: _this.fieldFormat,
            preventFocus: false,
        };
        _this.dialogRef = React.createRef();
        _this.fromDateRef = React.createRef();
        _this.isDaySelected = _this.isDaySelected.bind(_this);
        _this.isFirstDaySelected = _this.isFirstDaySelected.bind(_this);
        _this.isLastDaySelected = _this.isLastDaySelected.bind(_this);
        _this.isMiddleDaySelected = _this.isMiddleDaySelected.bind(_this);
        _this.onAccept = _this.onAccept.bind(_this);
        _this.onDaySelect = _this.onDaySelect.bind(_this);
        _this.onDialogDeactivate = _this.onDialogDeactivate.bind(_this);
        _this.onFromDateFieldChange = _this.onFromDateFieldChange.bind(_this);
        _this.onSetCustom = _this.onSetCustom.bind(_this);
        _this.onSetDay = _this.onSetDay.bind(_this);
        _this.onSetMonth = _this.onSetMonth.bind(_this);
        _this.onSetQuarter = _this.onSetQuarter.bind(_this);
        _this.onSetWeek = _this.onSetWeek.bind(_this);
        _this.onSetYear = _this.onSetYear.bind(_this);
        _this.onToDateFieldChange = _this.onToDateFieldChange.bind(_this);
        _this.openDialog = _this.openDialog.bind(_this);
        _this.setFromDateFocus = _this.setFromDateFocus.bind(_this);
        _this.setNextMonth = _this.setNextMonth.bind(_this);
        _this.setPreviousMonth = _this.setPreviousMonth.bind(_this);
        _this.defaultActions = (!_this.isInline && util_1.isCalendarActionRendered(_this.props)
            ? [{ type: definition_1.FieldActionTypesEnum.CALENDAR, onClick: _this.openDialog }]
            : []).concat(_this.defaultActions);
        return _this;
    }
    /**
     * @stable [07.03.2020]
     * @param {IDatesRangeEntity | DateTimeLikeTypeT | AnyT} currentRawValue
     */
    DateField.prototype.onChangeManually = function (currentRawValue) {
        var rangeEntity = currentRawValue;
        _super.prototype.onChangeManually.call(this, this.isRangeEnabled
            ? (this.fromDatesRangeEntity(__assign({}, rangeEntity, { from: this.serializeValue(rangeEntity.from), to: this.serializeValue(util_1.nvl(rangeEntity.to, rangeEntity.from)) })))
            : this.serializeValue(currentRawValue));
    };
    /**
     * @stable [09.01.2020]
     * @returns {JSX.Element}
     */
    DateField.prototype.getAttachmentElement = function () {
        var _this = this;
        var props = this.props;
        var dialogOpened = this.state.dialogOpened;
        var _a = props.dialogConfiguration, dialogConfiguration = _a === void 0 ? {} : _a;
        var className = util_1.joinClassName(!this.isInline && util_1.calc(dialogConfiguration.className), definition_1.ComponentClassesEnum.CALENDAR_BASE_DIALOG, this.isInline
            ? definition_1.ComponentClassesEnum.CALENDAR_INLINE_DIALOG
            : definition_1.ComponentClassesEnum.CALENDAR_DIALOG, this.isRangeEnabled
            ? definition_1.DateFieldClassesEnum.DATE_FIELD_CALENDARS_DIALOG
            : definition_1.DateFieldClassesEnum.DATE_FIELD_CALENDAR_DIALOG);
        if (this.isInline) {
            return (React.createElement("div", { className: className }, this.calendarAttachmentElement));
        }
        return util_1.orNull(dialogOpened, // To improve a performance
        function () { return (React.createElement(dialog_1.Dialog, __assign({ checkModal: true }, props.dialogConfiguration, { ref: _this.dialogRef, acceptable: false, closable: false, className: className, onDeactivate: _this.onDialogDeactivate }), _this.calendarAttachmentElement)); });
    };
    Object.defineProperty(DateField.prototype, "originalEmptyValue", {
        /**
         * @stable [07.03.2020]
         * @returns {StringNumberT[] | string}
         */
        get: function () {
            return this.isRangeEnabled ? [] : definition_1.FIELD_DISPLAY_EMPTY_VALUE;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [07.03.2020]
     * @param {IBaseEvent} event
     */
    DateField.prototype.onClick = function (event) {
        _super.prototype.onClick.call(this, event);
        if (!this.isInline && this.isFocusPrevented) {
            this.openDialog();
        }
    };
    /**
     * @stable [07.03.2020]
     * @param {DateFieldRangeValueT} value
     * @returns {string}
     */
    DateField.prototype.decorateDisplayValue = function (value) {
        if (this.isRangeEnabled) {
            var dateRangeEntity = this.fromDatesRangeValue(value);
            return R.isNil(dateRangeEntity)
                ? definition_1.FIELD_DISPLAY_EMPTY_VALUE
                : this.getDecoratedRangeDisplayValue(dateRangeEntity);
        }
        return this.serializeValue(value);
    };
    /**
     * @stable [09.03.2020]
     * @returns {string}
     */
    DateField.prototype.getFieldClassName = function () {
        return util_1.joinClassName(_super.prototype.getFieldClassName.call(this), definition_1.DateFieldClassesEnum.DATE_FIELD);
    };
    /**
     * @stable [09.03.2020]
     * @returns {string}
     */
    DateField.prototype.getFieldPattern = function () {
        var _this = this;
        return util_1.orNull(this.isFieldMaskNeeded, function () { return _super.prototype.getFieldPattern.call(_this) || _this.dateTimeSettings.uiDatePattern; });
    };
    /**
     * @stable [09.03.2020]
     * @returns {Array<string | RegExp>}
     */
    DateField.prototype.getFieldMask = function () {
        var _this = this;
        return util_1.orNull(this.isFieldMaskNeeded, function () { return _super.prototype.getFieldMask.call(_this) || _this.dateTimeSettings.uiDateMask; });
    };
    Object.defineProperty(DateField.prototype, "fieldFormat", {
        /**
         * @stable [09.03.2020]
         * @returns {string}
         */
        get: function () {
            return util_1.nvl(this.props.format, this.dateTimeSettings.uiDateFormat);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "calendarAttachmentElement", {
        /**
         * @stable [22.01.2020]
         * @returns {JSX.Element}
         */
        get: function () {
            return (React.createElement(React.Fragment, null,
                this.isRangeEnabled && this.quickButtonsElement,
                React.createElement("div", { className: 'rac-calendar-dialog__range-explorer' },
                    React.createElement(button_1.Button, { icon: 'back', mini: true, onClick: this.setPreviousMonth }),
                    React.createElement("div", { className: 'rac-calendar-dialog__range-explorer-date' }, this.rangeExplorerDateElement),
                    React.createElement(button_1.Button, { icon: 'forward', mini: true, onClick: this.setNextMonth })),
                this.calendarElement,
                React.createElement("div", { className: 'rac-calendar-dialog__footer' },
                    this.dateFieldsElement,
                    !this.isInline && (React.createElement(button_1.Button, { text: this.settings.messages.OK, onClick: this.onAccept })))));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "calendarElement", {
        /**
         * @stable [20.01.2020]
         * @returns {JSX.Element}
         */
        get: function () {
            var defaultCalendarProps = this.defaultCalendarProps;
            var _a = defaultCalendarProps.gridConfiguration, gridConfiguration = _a === void 0 ? {} : _a;
            if (this.isRangeEnabled) {
                return (React.createElement("div", { className: 'rac-calendar-dialog__calendars-wrapper' },
                    React.createElement(calendar_1.Calendar, __assign({}, defaultCalendarProps, { calendarEntity: this.currentCalendarEntity })),
                    React.createElement(calendar_1.Calendar, __assign({}, defaultCalendarProps, { gridConfiguration: __assign({}, gridConfiguration, { wrapperClassName: util_1.joinClassName('rac-calendar-dialog__second-calendar', util_1.calc(gridConfiguration.className)) }), calendarEntity: this.nextCalendarEntity }))));
            }
            return (React.createElement(calendar_1.Calendar, __assign({}, defaultCalendarProps, { calendarEntity: this.currentCalendarEntity })));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "defaultCalendarProps", {
        /**
         * @stable [21.01.2020]
         * @returns {ICalendarProps}
         */
        get: function () {
            return __assign({ gridConfiguration: { headerRendered: true }, showOnlyCurrentDays: true }, this.props.calendarConfiguration, { isSelected: this.isDaySelected, isMiddleSelected: this.isMiddleDaySelected, isFirstSelected: this.isFirstDaySelected, isLastSelected: this.isLastDaySelected, onSelect: this.onDaySelect });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "rangeExplorerDateElement", {
        /**
         * @stable [20.01.2020]
         * @returns {React.ReactNode}
         */
        get: function () {
            if (this.isRangeEnabled) {
                return (React.createElement(React.Fragment, null,
                    React.createElement("div", { className: 'rac-calendar-dialog__range-explorer-date-content' }, this.currentDateAsDisplayValue),
                    React.createElement("div", { className: 'rac-calendar-dialog__range-explorer-date-content rac-calendar-dialog__range-explorer-date-content-next' }, this.nextDateAsDisplayValue)));
            }
            return this.currentDateAsDisplayValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "dateFieldsElement", {
        /**
         * @stable [20.01.2020]
         * @returns {JSX.Element}
         */
        get: function () {
            return (React.createElement(React.Fragment, null,
                React.createElement(DateField, __assign({}, this.defaultRangeFieldProps, { autoFocus: true, ref: this.fromDateRef, value: this.state.fromDate, onChange: this.onFromDateFieldChange })),
                this.isRangeEnabled
                    && ![this.selectedPeriodMode, this.aheadEntityPeriodMode].includes(definition_1.DatePeriodsEnum.DAY)
                    && (React.createElement(React.Fragment, null,
                        React.createElement("span", { className: definition_1.ComponentClassesEnum.CALENDAR_DIALOG_RANGE_INPUT_SEPARATOR }, "\u2014"),
                        React.createElement(DateField, __assign({}, this.defaultRangeFieldProps, { value: this.state.toDate, onChange: this.onToDateFieldChange }))))));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "quickButtonsElement", {
        /**
         * @stable [07.03.2020]
         * @returns {JSX.Element}
         */
        get: function () {
            var previousPeriodModeEnabled = this.isPreviousPeriodModeEnabled;
            var selectedPeriodMode = this.selectedPeriodMode;
            var aheadEntityPeriodMode = this.aheadEntityPeriodMode;
            var buttonsElement = util_1.objectValuesArrayFilter((!previousPeriodModeEnabled || aheadEntityPeriodMode === definition_1.DatePeriodsEnum.DAY) && (React.createElement(button_1.Button, { key: 'quick-action-day', full: true, className: util_1.joinClassName(selectedPeriodMode === definition_1.DatePeriodsEnum.DAY && definition_1.CalendarDialogClassesEnum.SELECTED_QUICK_ACTION), text: previousPeriodModeEnabled
                    ? this.settings.messages.PREVIOUS_DAY
                    : this.settings.messages.DAY, onClick: this.onSetDay })), (!previousPeriodModeEnabled || aheadEntityPeriodMode === definition_1.DatePeriodsEnum.WEEK) && (React.createElement(button_1.Button, { key: 'quick-action-week', full: true, className: util_1.joinClassName(selectedPeriodMode === definition_1.DatePeriodsEnum.WEEK && definition_1.CalendarDialogClassesEnum.SELECTED_QUICK_ACTION), text: previousPeriodModeEnabled
                    ? this.settings.messages.PREVIOUS_WEEK
                    : this.settings.messages.WEEK, onClick: this.onSetWeek })), (!previousPeriodModeEnabled || aheadEntityPeriodMode === definition_1.DatePeriodsEnum.MONTH) && (React.createElement(button_1.Button, { key: 'quick-action-month', full: true, className: util_1.joinClassName(selectedPeriodMode === definition_1.DatePeriodsEnum.MONTH && definition_1.CalendarDialogClassesEnum.SELECTED_QUICK_ACTION), text: previousPeriodModeEnabled
                    ? this.settings.messages.PREVIOUS_MONTH
                    : this.settings.messages.MONTH, onClick: this.onSetMonth })), (!previousPeriodModeEnabled || aheadEntityPeriodMode === definition_1.DatePeriodsEnum.QUARTER) && (React.createElement(button_1.Button, { key: 'quick-action-quarter', full: true, className: util_1.joinClassName(selectedPeriodMode === definition_1.DatePeriodsEnum.QUARTER && definition_1.CalendarDialogClassesEnum.SELECTED_QUICK_ACTION), text: previousPeriodModeEnabled
                    ? this.settings.messages.PREVIOUS_QUARTER
                    : this.settings.messages.QUARTER, onClick: this.onSetQuarter })), (!previousPeriodModeEnabled || aheadEntityPeriodMode === definition_1.DatePeriodsEnum.YEAR) && (React.createElement(button_1.Button, { key: 'quick-action-year', full: true, className: util_1.joinClassName(selectedPeriodMode === definition_1.DatePeriodsEnum.YEAR && definition_1.CalendarDialogClassesEnum.SELECTED_QUICK_ACTION), text: previousPeriodModeEnabled
                    ? this.settings.messages.PREVIOUS_YEAR
                    : this.settings.messages.YEAR, onClick: this.onSetYear })), !previousPeriodModeEnabled && (React.createElement(button_1.Button, { key: 'quick-action-custom', full: true, className: util_1.joinClassName(selectedPeriodMode === definition_1.DatePeriodsEnum.CUSTOM && definition_1.CalendarDialogClassesEnum.SELECTED_QUICK_ACTION), text: this.settings.messages.CUSTOM, onClick: this.onSetCustom })));
            if (R.isEmpty(buttonsElement)) {
                return null;
            }
            return (React.createElement("div", { className: 'rac-calendar-dialog__quick-buttons' }, buttonsElement));
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [07.03.2020]
     */
    DateField.prototype.onSetDay = function () {
        var currentDate = this.dc.getCurrentDate();
        if (this.isPreviousPeriodModeEnabled) {
            var previousDay = this.dc.addDaysAsDate({ date: this.aheadEntity.from, duration: -1 });
            this.setQuickValue(previousDay, previousDay, definition_1.DatePeriodsEnum.DAY);
        }
        else {
            this.setQuickValue(currentDate, currentDate, definition_1.DatePeriodsEnum.DAY);
        }
    };
    /**
     * @stable [07.03.2020]
     */
    DateField.prototype.onSetWeek = function () {
        if (this.isPreviousPeriodModeEnabled) {
            var aheadEntity = this.aheadEntity;
            var firstDayOfWeekAsDate = this.dc.asFirstDayOfWeekAsDate({ date: aheadEntity.from, inputFormat: this.fieldFormat });
            var firstDayOfPreviousWeek = this.dc.addWeeksAsDate({ date: firstDayOfWeekAsDate, duration: -1 });
            this.setQuickValue(firstDayOfPreviousWeek, this.dc.asLastDayOfWeekAsDate({ date: firstDayOfPreviousWeek }), definition_1.DatePeriodsEnum.WEEK);
        }
        else {
            this.setQuickValue(this.dc.asFirstDayOfWeekAsDate(), this.dc.asLastDayOfWeekAsDate(), definition_1.DatePeriodsEnum.WEEK);
        }
    };
    /**
     * @stable [07.03.2020]
     */
    DateField.prototype.onSetMonth = function () {
        if (this.isPreviousPeriodModeEnabled) {
            var aheadEntity = this.aheadEntity;
            var firstDayOfMonthAsDate = this.dc.asFirstDayOfMonthAsDate({ date: aheadEntity.from, inputFormat: this.fieldFormat });
            var firstDayOfPreviousMonth = this.dc.addMonthsAsDate({ date: firstDayOfMonthAsDate, duration: -1 });
            this.setQuickValue(firstDayOfPreviousMonth, this.dc.asLastDayOfMonthAsDate({ date: firstDayOfPreviousMonth }), definition_1.DatePeriodsEnum.MONTH);
        }
        else {
            this.setQuickValue(this.dc.asFirstDayOfMonthAsDate(), this.dc.asLastDayOfMonthAsDate(), definition_1.DatePeriodsEnum.MONTH);
        }
    };
    /**
     * @stable [08.03.2020]
     */
    DateField.prototype.onSetQuarter = function () {
        if (this.isPreviousPeriodModeEnabled) {
            var aheadEntity = this.aheadEntity;
            var firstDayOfQuarterAsDate = this.dc.asFirstDayOfQuarterAsDate({ date: aheadEntity.from, inputFormat: this.fieldFormat });
            var firstDayOfPreviousQuarter = this.dc.addQuartersAsDate({ date: firstDayOfQuarterAsDate, duration: -1 });
            this.setQuickValue(firstDayOfPreviousQuarter, this.dc.asLastDayOfQuarterAsDate({ date: firstDayOfPreviousQuarter }), definition_1.DatePeriodsEnum.QUARTER);
        }
        else {
            this.setQuickValue(this.dc.asFirstDayOfQuarterAsDate(), this.dc.asLastDayOfQuarterAsDate(), definition_1.DatePeriodsEnum.QUARTER);
        }
    };
    /**
     * @stable [08.03.2020]
     */
    DateField.prototype.onSetYear = function () {
        if (this.isPreviousPeriodModeEnabled) {
            var aheadEntity = this.aheadEntity;
            var firstDayOfYearAsDate = this.dc.asFirstDayOfYearAsDate({ date: aheadEntity.from, inputFormat: this.fieldFormat });
            var firstDayOfPreviousYear = this.dc.addYearsAsDate({ date: firstDayOfYearAsDate, duration: -1 });
            this.setQuickValue(firstDayOfPreviousYear, this.dc.asLastDayOfYearAsDate({ date: firstDayOfPreviousYear }), definition_1.DatePeriodsEnum.YEAR);
        }
        else {
            this.setQuickValue(this.dc.asFirstDayOfYearAsDate(), this.dc.asLastDayOfYearAsDate(), definition_1.DatePeriodsEnum.YEAR);
        }
    };
    /**
     * @stable [08.03.2020]
     */
    DateField.prototype.onSetCustom = function () {
        this.setState({
            cursor: definitions_interface_1.UNDEF,
            from: null,
            periodMode: definition_1.DatePeriodsEnum.CUSTOM,
            to: null,
        });
    };
    /**
     * @stable [06.03.2020]
     * @param {Date} from
     * @param {Date} to
     * @param {DatePeriodsEnum} periodMode
     */
    DateField.prototype.setQuickValue = function (from, to, periodMode) {
        this.setState(__assign({ cursor: from, from: from,
            periodMode: periodMode,
            to: to }, ([definition_1.DatePeriodsEnum.DAY].includes(periodMode) ? { toDate: definitions_interface_1.UNDEF } : {})), this.setFromDateFocus);
    };
    /**
     * @stable [11.03.2020]
     */
    DateField.prototype.setFromDateFocus = function () {
        util_1.ifNotNilThanValue(this.fromDateRef.current, function (field) { return field.setFocus(); });
    };
    /**
     * @stable [07.03.2020]
     */
    DateField.prototype.setPreviousMonth = function () {
        this.doChangeMonth(-1);
    };
    /**
     * @stable [07.03.2020]
     */
    DateField.prototype.setNextMonth = function () {
        this.doChangeMonth(1);
    };
    /**
     * @stable [07.03.2020]
     * @param {number} duration
     */
    DateField.prototype.doChangeMonth = function (duration) {
        this.setState({ cursor: this.dc.addMonthsAsDate({ date: this.currentCalendarDate, duration: duration }) });
    };
    /**
     * @stable [09.03.2020]
     * @param {string} value
     */
    DateField.prototype.onToDateFieldChange = function (value) {
        var to = this.getValueAsDate(value);
        this.setState(R.isNil(to)
            ? { toDate: value }
            : ({ periodMode: definition_1.DatePeriodsEnum.CUSTOM, to: to, toDate: value }));
    };
    /**
     * @stable [11.03.2020]
     * @param {string} value
     */
    DateField.prototype.onFromDateFieldChange = function (value) {
        var from = this.getValueAsDate(value);
        this.setState(R.isNil(from)
            ? { fromDate: value }
            : (__assign({ cursor: from, from: from, fromDate: value }, (this.selectedPeriodMode === definition_1.DatePeriodsEnum.DAY
                ? { to: null, toDate: definitions_interface_1.UNDEF }
                : { periodMode: definition_1.DatePeriodsEnum.CUSTOM }))));
    };
    /**
     * @stable [10.03.2020]
     */
    DateField.prototype.onAccept = function () {
        var _this = this;
        var acceptedValue = this.isRangeEnabled
            ? this.acceptedDatesRangeEntity
            : this.acceptedDateValue;
        var acceptedValueAsRangeEntity = acceptedValue;
        this.onDialogClose(function () {
            util_1.ifNotEmptyThanValue(acceptedValue, function () {
                if (!_this.isRangeEnabled || util_1.isObjectNotEmpty(acceptedValueAsRangeEntity.from)) {
                    _this.onChangeManually(acceptedValue);
                }
            });
            _this.setFocus();
        });
    };
    /**
     * @stable [09.03.2020]
     * @param {ICalendarDayEntity} calendarDayEntity
     */
    DateField.prototype.onDaySelect = function (calendarDayEntity) {
        var _this = this;
        var updatedState;
        if (this.isRangeEnabled) {
            var periodMode_1 = this.isPreviousPeriodModeEnabled
                ? this.aheadEntityPeriodMode
                : this.selectedPeriodMode;
            var rangeValues_1 = {};
            var result = this.dc.selectDaysOfYearRange(this.currentFromToDayOfYearEntity, calendarDayEntity);
            var rangeValuesFrom = util_1.ifNotEmptyThanValue(result.from, function (from) { return ({ from: _this.dc.fromDayOfYearEntityAsDate(from) }); });
            switch (periodMode_1) {
                case definition_1.DatePeriodsEnum.DAY:
                case definition_1.DatePeriodsEnum.WEEK:
                case definition_1.DatePeriodsEnum.MONTH:
                case definition_1.DatePeriodsEnum.QUARTER:
                case definition_1.DatePeriodsEnum.YEAR:
                    util_1.ifNotEmptyThanValue(rangeValuesFrom.from, function (from) {
                        switch (periodMode_1) {
                            case definition_1.DatePeriodsEnum.DAY:
                                rangeValues_1 = {
                                    from: from,
                                    to: from,
                                };
                                break;
                            case definition_1.DatePeriodsEnum.WEEK:
                                rangeValues_1 = {
                                    from: _this.dc.asFirstDayOfWeekAsDate({ date: from }),
                                    to: _this.dc.asLastDayOfWeekAsDate({ date: from }),
                                };
                                break;
                            case definition_1.DatePeriodsEnum.MONTH:
                                rangeValues_1 = {
                                    from: _this.dc.asFirstDayOfMonthAsDate({ date: from }),
                                    to: _this.dc.asLastDayOfMonthAsDate({ date: from }),
                                };
                                break;
                            case definition_1.DatePeriodsEnum.QUARTER:
                                rangeValues_1 = {
                                    from: _this.dc.asFirstDayOfQuarterAsDate({ date: from }),
                                    to: _this.dc.asLastDayOfQuarterAsDate({ date: from }),
                                };
                                break;
                            case definition_1.DatePeriodsEnum.YEAR:
                                rangeValues_1 = {
                                    from: _this.dc.asFirstDayOfYearAsDate({ date: from }),
                                    to: _this.dc.asLastDayOfYearAsDate({ date: from }),
                                };
                                break;
                        }
                    });
                    break;
                case definition_1.DatePeriodsEnum.CUSTOM:
                    rangeValues_1 = __assign({}, rangeValuesFrom, util_1.ifNotEmptyThanValue(result.to, function (to) { return ({ to: _this.dc.fromDayOfYearEntityAsDate(to) }); }));
                    break;
            }
            updatedState = __assign({ cursor: definitions_interface_1.UNDEF, from: null, fromDate: definitions_interface_1.UNDEF, to: null, toDate: definitions_interface_1.UNDEF }, rangeValues_1, (this.isPreviousPeriodModeEnabled ? { periodMode: null } : { periodMode: periodMode_1 }));
            util_1.ifNotEmptyThanValue(updatedState.from, function () { return (updatedState.fromDate = _this.serializeValue(updatedState.from)); });
            util_1.ifNotEmptyThanValue(updatedState.to, function () { return (updatedState.toDate = _this.serializeValue(updatedState.to)); });
        }
        else {
            updatedState = {
                cursor: definitions_interface_1.UNDEF,
                from: calendarDayEntity.date,
                fromDate: definitions_interface_1.UNDEF,
            };
            util_1.ifNotEmptyThanValue(updatedState.from, function () { return (updatedState.fromDate = _this.serializeValue(calendarDayEntity.date)); });
        }
        this.setState(updatedState, this.setFromDateFocus);
    };
    /**
     * @stable [08.03.2020]
     */
    DateField.prototype.onDialogDeactivate = function () {
        this.onDialogClose();
    };
    /**
     * @stable [08.03.2020]
     * @param {() => void} callback
     */
    DateField.prototype.onDialogClose = function (callback) {
        this.setState({
            cursor: definitions_interface_1.UNDEF,
            dialogOpened: definitions_interface_1.UNDEF,
            from: definitions_interface_1.UNDEF,
            fromDate: definitions_interface_1.UNDEF,
            periodMode: definitions_interface_1.UNDEF,
            to: definitions_interface_1.UNDEF,
            toDate: definitions_interface_1.UNDEF,
        }, callback);
    };
    /**
     * @stable [08.03.2020]
     */
    DateField.prototype.openDialog = function () {
        var _this = this;
        this.setState({ dialogOpened: true }, function () { return _this.dialog.activate(); });
    };
    /**
     * @stable [09.03.2020]
     * @param {IDatesRangeEntity} dateRangeEntity
     * @returns {string}
     */
    DateField.prototype.getDecoratedRangeDisplayValue = function (dateRangeEntity) {
        var dateFromAsString = this.serializeValue(dateRangeEntity.from);
        var dateToAsString = this.serializeValue(dateRangeEntity.to);
        var messages = this.settings.messages;
        if (this.checkRange(dateFromAsString, dateToAsString, this.dc.addDaysAsDate({ date: this.dc.getCurrentDate(), duration: -1 }), this.dc.addDaysAsDate({ date: this.dc.getCurrentDate(), duration: -1 }))) {
            return messages.YESTERDAY + ": " + this.dc.dateAsPstDateString({ date: dateRangeEntity.from });
        }
        else if (this.checkRange(dateFromAsString, dateToAsString, this.dc.addWeeksAsDate({ date: this.dc.asFirstDayOfWeekAsDate(), duration: -1 }), this.dc.addWeeksAsDate({ date: this.dc.asLastDayOfWeekAsDate(), duration: -1 }))) {
            return this.getDecoratedRangeSpecificValue(dateRangeEntity.from, dateRangeEntity.to, messages.PREVIOUS_WEEK);
        }
        else if (this.checkRange(dateFromAsString, dateToAsString, this.dc.addMonthsAsDate({ date: this.dc.asFirstDayOfMonthAsDate(), duration: -1 }), this.dc.addMonthsAsDate({ date: this.dc.asLastDayOfMonthAsDate(), duration: -1 }))) {
            return this.getDecoratedRangeSpecificValue(dateRangeEntity.from, dateRangeEntity.to, messages.PREVIOUS_MONTH);
        }
        else if (this.checkRange(dateFromAsString, dateToAsString, this.dc.addQuartersAsDate({ date: this.dc.asFirstDayOfQuarterAsDate(), duration: -1 }), this.dc.addQuartersAsDate({ date: this.dc.asLastDayOfQuarterAsDate(), duration: -1 }))) {
            return this.getDecoratedRangeSpecificValue(dateRangeEntity.from, dateRangeEntity.to, messages.PREVIOUS_QUARTER);
        }
        else if (this.checkRange(dateFromAsString, dateToAsString, this.dc.addYearsAsDate({ date: this.dc.asFirstDayOfYearAsDate(), duration: -1 }), this.dc.addYearsAsDate({ date: this.dc.asLastDayOfYearAsDate(), duration: -1 }))) {
            return this.getDecoratedRangeSpecificValue(dateRangeEntity.from, dateRangeEntity.to, messages.PREVIOUS_YEAR);
        }
        else if (this.checkRange(dateFromAsString, dateToAsString, this.dc.getCurrentDate(), this.dc.getCurrentDate())) {
            return messages.TODAY + ": " + this.dc.dateAsPstDateString({ date: dateRangeEntity.from });
        }
        else if (this.checkRange(dateFromAsString, dateToAsString, this.dc.asFirstDayOfWeekAsDate(), this.dc.asLastDayOfWeekAsDate())) {
            return this.getDecoratedRangeSpecificValue(dateRangeEntity.from, dateRangeEntity.to, messages.THIS_WEEK);
        }
        else if (this.checkRange(dateFromAsString, dateToAsString, this.dc.asFirstDayOfMonthAsDate(), this.dc.asLastDayOfMonthAsDate())) {
            return this.getDecoratedRangeSpecificValue(dateRangeEntity.from, dateRangeEntity.to, messages.THIS_MONTH);
        }
        else if (this.checkRange(dateFromAsString, dateToAsString, this.dc.asFirstDayOfQuarterAsDate(), this.dc.asLastDayOfQuarterAsDate())) {
            return this.getDecoratedRangeSpecificValue(dateRangeEntity.from, dateRangeEntity.to, messages.THIS_QUARTER);
        }
        else if (this.checkRange(dateFromAsString, dateToAsString, this.dc.asFirstDayOfYearAsDate(), this.dc.asLastDayOfYearAsDate())) {
            return this.getDecoratedRangeSpecificValue(dateRangeEntity.from, dateRangeEntity.to, messages.THIS_YEAR);
        }
        if (R.equals(dateToAsString, dateFromAsString)) {
            return messages.DATE + ": " + dateFromAsString;
        }
        return dateFromAsString + " " + definitions_interface_1.UniCodesEnum.N_DASH + " " + dateToAsString;
    };
    /**
     * @stable [09.03.2020]
     * @param {DateTimeLikeTypeT} from
     * @param {DateTimeLikeTypeT} to
     * @param {string} label
     * @returns {string}
     */
    DateField.prototype.getDecoratedRangeSpecificValue = function (from, to, label) {
        var dateFrom = this.getValueAsDate(from);
        var dateTo = this.getValueAsDate(to);
        return label + ": " + this.dc.dateAsPstDateString({ date: dateFrom }) + " " + definitions_interface_1.UniCodesEnum.N_DASH + " " + this.dc.dateAsPstDateString({ date: dateTo });
    };
    /**
     * @stable [09.03.2020]
     * @param {string} dateFromAsString
     * @param {string} dateToAsString
     * @param {Date} dateFrom
     * @param {Date} dateTo
     * @returns {boolean}
     */
    DateField.prototype.checkRange = function (dateFromAsString, dateToAsString, dateFrom, dateTo) {
        return this.serializeValue(dateFrom) === dateFromAsString &&
            this.serializeValue(dateTo) === dateToAsString;
    };
    /**
     * @stable [08.03.2020]
     * @param {DateTimeLikeTypeT} value
     * @returns {string}
     */
    DateField.prototype.serializeValue = function (value) {
        return this.dc.dateAsString({ date: value, inputFormat: this.fieldFormat, outputFormat: this.fieldFormat });
    };
    Object.defineProperty(DateField.prototype, "currentDateAsDisplayValue", {
        /**
         * @stable [08.03.2020]
         * @returns {string}
         */
        get: function () {
            return this.dc.dateAsString({ date: this.currentCalendarDate, outputFormat: this.props.headerFormat });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "nextDateAsDisplayValue", {
        /**
         * @stable [08.03.2020]
         * @returns {string}
         */
        get: function () {
            return this.dc.dateAsString({ date: this.nextCalendarDate, outputFormat: this.props.headerFormat });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "currentCalendarEntity", {
        /**
         * @stable [08.03.2020]
         * @returns {ICalendarEntity}
         */
        get: function () {
            return this.getCalendarEntity(this.currentCalendarDate);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "nextCalendarEntity", {
        /**
         * @stable [08.03.2020]
         * @returns {ICalendarEntity}
         */
        get: function () {
            return this.getCalendarEntity(this.nextCalendarDate);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [08.03.2020]
     * @param {Date} date
     * @returns {ICalendarEntity}
     */
    DateField.prototype.getCalendarEntity = function (date) {
        return this.dc.asCalendar(__assign({}, this.props.calendarEntityConfiguration, { date: date }));
    };
    Object.defineProperty(DateField.prototype, "currentCalendarDate", {
        /**
         * @stable [07.03.2020]
         * @returns {Date}
         */
        get: function () {
            var _a = this.state, cursor = _a.cursor, from = _a.from;
            return cursor
                || (this.isRangeEnabled
                    ? (from || this.getValueAsDate(this.valueAsDateFrom))
                    : this.acceptedDateValue)
                || this.dc.getCurrentDate();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "nextCalendarDate", {
        /**
         * @stable [08.03.2020]
         * @returns {Date}
         */
        get: function () {
            return this.dc.addMonthsAsDate({ date: this.currentCalendarDate, duration: 1 });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "currentFromToDayOfYearEntity", {
        /**
         * @stable [07.03.2020]
         * @returns {IFromToDayOfYearEntity}
         */
        get: function () {
            var _this = this;
            var _a = this.state, from = _a.from, to = _a.to;
            var result = __assign({}, (util_1.isDef(from) // "Null" as value (!)
                ? ({ from: util_1.ifNotNilThanValue(from, function () { return _this.dc.asDayOfYearEntity({ date: from }); }) })
                : ({})), (util_1.isDef(to)
                ? ({ to: util_1.ifNotNilThanValue(to, function () { return _this.dc.asDayOfYearEntity({ date: to }); }) })
                : ({})));
            if (util_1.isUndef(result.from) || util_1.isUndef(result.to)) {
                var rangeEntity = this.datesRangeEntity;
                if (!R.isNil(rangeEntity)) {
                    if (util_1.isUndef(result.from) && !R.isNil(rangeEntity.from)) {
                        result.from = this.dc.asDayOfYearEntity({ date: rangeEntity.from, inputFormat: this.fieldFormat });
                    }
                    if (util_1.isUndef(result.to) && !R.isNil(rangeEntity.to)) {
                        result.to = this.dc.asDayOfYearEntity({ date: rangeEntity.to, inputFormat: this.fieldFormat });
                    }
                }
            }
            return util_1.notNilValuesFilter(result);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [07.03.2020]
     * @param {IDatesRangeEntity} entity
     * @returns {DatesRangeValueT}
     */
    DateField.prototype.fromDatesRangeEntity = function (entity) {
        return this.fieldConverter.convert({
            from: definition_1.FieldConverterTypesEnum.DATES_RANGE_ENTITY,
            to: definition_1.FieldConverterTypesEnum.DATES_RANGE_VALUE,
            value: entity,
        });
    };
    /**
     * @stable [07.03.2020]
     * @param {DatesRangeValueT} value
     * @returns {IDatesRangeEntity}
     */
    DateField.prototype.fromDatesRangeValue = function (value) {
        return this.fieldConverter.convert({
            from: definition_1.FieldConverterTypesEnum.DATES_RANGE_VALUE,
            to: definition_1.FieldConverterTypesEnum.DATES_RANGE_ENTITY,
            value: value,
        });
    };
    Object.defineProperty(DateField.prototype, "datesRangeEntity", {
        /**
         * @stable [07.03.2020]
         * @returns {IDatesRangeEntity}
         */
        get: function () {
            return this.fromDatesRangeValue(this.value);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [06.03.2020]
     * @param {DateTimeLikeTypeT} date
     * @returns {Date}
     */
    DateField.prototype.getValueAsDate = function (date) {
        return this.dc.asDate({ date: date, inputFormat: this.fieldFormat });
    };
    Object.defineProperty(DateField.prototype, "acceptedDatesRangeEntity", {
        /**
         * @stable [10.03.2020]
         * @returns {IDatesRangeEntity}
         */
        get: function () {
            var _a = this.state, from = _a.from, periodMode = _a.periodMode, to = _a.to;
            var selectedPeriodMode = this.selectedPeriodMode;
            if (![selectedPeriodMode].includes(definition_1.DatePeriodsEnum.DAY)
                && (util_1.isObjectNotEmpty(from) || util_1.isObjectNotEmpty(to))) {
                return util_1.defValuesFilter({
                    from: from || this.valueAsDateFrom,
                    periodMode: periodMode,
                    to: to || this.valueAsDateTo,
                });
            }
            return util_1.defValuesFilter({ from: from, periodMode: periodMode, to: to });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "acceptedDateValue", {
        /**
         * @stable [06.03.2020]
         * @returns {Date}
         */
        get: function () {
            return this.state.from || this.valueAsDate;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "valueAsDate", {
        /**
         * @stable [06.03.2020]
         * @returns {Date}
         */
        get: function () {
            return this.getValueAsDate(this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "valueAsPeriodMode", {
        /**
         * @stable [06.03.2020]
         * @returns {DatePeriodsEnum}
         */
        get: function () {
            return util_1.ifNotNilThanValue(this.datesRangeEntity, function (rangeEntity) { return rangeEntity.periodMode; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "valueAsDateFrom", {
        /**
         * @stable [08.03.2020]
         * @returns {DatePeriodsEnum}
         */
        get: function () {
            return util_1.ifNotNilThanValue(this.datesRangeEntity, function (rangeEntity) { return rangeEntity.from; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "valueAsDateTo", {
        /**
         * @stable [09.03.2020]
         * @returns {DateTimeLikeTypeT}
         */
        get: function () {
            return util_1.ifNotNilThanValue(this.datesRangeEntity, function (rangeEntity) { return rangeEntity.to; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "aheadValue", {
        /**
         * @stable [06.03.2020]
         * @returns {DateFieldRangeValueT}
         */
        get: function () {
            return this.props.aheadValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "aheadEntity", {
        /**
         * @stable [07.03.2020]
         * @returns {IDatesRangeEntity}
         */
        get: function () {
            return this.fromDatesRangeValue(this.aheadValue);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "aheadEntityPeriodMode", {
        /**
         * @stable [07.03.2020]
         * @returns {DatePeriodsEnum}
         */
        get: function () {
            return util_1.ifNotNilThanValue(this.aheadEntity, function (aheadEntity) { return aheadEntity.periodMode; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "selectedPeriodMode", {
        /**
         * @stable [07.03.2020]
         * @returns {DatePeriodsEnum}
         */
        get: function () {
            var periodMode = this.state.periodMode;
            return util_1.isDef(periodMode) ? periodMode : util_1.nvl(this.valueAsPeriodMode, definition_1.DatePeriodsEnum.CUSTOM);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [07.03.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    DateField.prototype.isDaySelected = function (entity) {
        return this.isRangeEnabled
            ? this.isRangeSelected(entity)
            : this.dc.compare(entity.date, this.acceptedDateValue) === 0;
    };
    /**
     * @stable [08.03.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    DateField.prototype.isRangeSelected = function (entity) {
        return this.dc.isDayOfYearBelongToDaysOfYearRange(this.currentFromToDayOfYearEntity, entity);
    };
    /**
     * @stable [07.03.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    DateField.prototype.isFirstDaySelected = function (entity) {
        var currentFromToDayOfYearEntity = this.currentFromToDayOfYearEntity;
        return (this.isServiceDaySelected(currentFromToDayOfYearEntity)
            && util_1.isObjectNotEmpty(currentFromToDayOfYearEntity.to)
            && (this.dc.isDayOfYearBelongToDaysOfYearRange(__assign({}, currentFromToDayOfYearEntity, { to: {} }), entity)));
    };
    /**
     * @stable [07.03.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    DateField.prototype.isLastDaySelected = function (entity) {
        var currentFromToDayOfYearEntity = this.currentFromToDayOfYearEntity;
        return (this.isServiceDaySelected(currentFromToDayOfYearEntity)
            && util_1.isObjectNotEmpty(currentFromToDayOfYearEntity.from)
            && (this.dc.isDayOfYearBelongToDaysOfYearRange(__assign({}, currentFromToDayOfYearEntity, { from: {} }), entity)));
    };
    /**
     * @stable [07.03.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    DateField.prototype.isMiddleDaySelected = function (entity) {
        var currentFromToDayOfYearEntity = this.currentFromToDayOfYearEntity;
        return (this.isServiceDaySelected(currentFromToDayOfYearEntity) && (!this.isLastDaySelected(entity)
            && !this.isFirstDaySelected(entity)
            && util_1.isObjectNotEmpty(currentFromToDayOfYearEntity.to)
            && util_1.isObjectNotEmpty(currentFromToDayOfYearEntity.from)));
    };
    /**
     * @stable [07.03.2020]
     * @param {IFromToDayOfYearEntity} currentFromToDayOfYearEntity
     * @returns {boolean}
     */
    DateField.prototype.isServiceDaySelected = function (currentFromToDayOfYearEntity) {
        return (this.isRangeEnabled
            && !this.dc.isDayOfYearEqualOtherDayOfYear(currentFromToDayOfYearEntity.from, currentFromToDayOfYearEntity.to));
    };
    Object.defineProperty(DateField.prototype, "isPreviousPeriodModeEnabled", {
        /**
         * @stable [06.03.2020]
         * @returns {boolean}
         */
        get: function () {
            return !R.isNil(this.aheadValue);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "isRangeEnabled", {
        /**
         * @stable [06.03.2020]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isRangeEnabled(this.props);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "isInline", {
        /**
         * @stable [06.03.2020]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isInline(this.props);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "isFieldMaskNeeded", {
        /**
         * @stable [09.03.2020]
         * @returns {boolean}
         */
        get: function () {
            return !this.isDisplayValueDefined && !this.isRangeEnabled;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "dateTimeSettings", {
        /**
         * stable [06.03.2020]
         * @returns {IDateTimeSettingsEntity}
         */
        get: function () {
            return this.settings.dateTime || {};
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "dialog", {
        /**
         * @stable [06.03.2020]
         * @returns {Dialog}
         */
        get: function () {
            return this.dialogRef.current;
        },
        enumerable: true,
        configurable: true
    });
    DateField.defaultProps = {
        headerFormat: 'MMMM YYYY',
        preventFocus: true,
    };
    return DateField;
}(text_field_1.BaseTextField));
exports.DateField = DateField;
//# sourceMappingURL=date-field.component.js.map