import * as React from 'react';
import { ILogger } from 'ts-smart-logger';
import { IUniversalField } from '../../../entities-definitions.interface';
import { IUniversalFieldProps } from '../../../configurations-definitions.interface';
import { AnyT, IFocusEvent, IKeyboardEvent } from '../../../definitions.interface';
import { IUniversalFieldState } from './field.interface';
import { IBaseEvent } from '../../../definition';
import { UniversalComponent } from '../../base/universal.component';
export declare abstract class UniversalField<TProps extends IUniversalFieldProps, TState extends IUniversalFieldState> extends UniversalComponent<TProps, TState> implements IUniversalField<TProps, TState> {
    protected static logger: ILogger;
    private static DEFAULT_CARET_BLINKING_FREQUENCY_TIMEOUT;
    private caretBlinkingTask;
    /**
     * @stable [17.06.2018]
     * @param {TProps} props
     */
    constructor(props: TProps);
    /**
     * @stable [20.08.2018]
     */
    componentDidMount(): void;
    /**
     * @stable [04.09.2018]
     * @param {TProps} prevProps
     * @param {TState} prevState
     */
    componentDidUpdate(prevProps: TProps, prevState: TState): void;
    /**
     * @stable [17.06.2018]
     */
    resetError(): void;
    /**
     * @stable [20.08.2018]
     */
    clearValue(): void;
    /**
     * @stable [11.01.2020]
     * @param {IBaseEvent} event
     */
    onChange(event: IBaseEvent): void;
    /**
     * @stable [03.02.2020]
     * @param {TValue} currentRawValue
     */
    onChangeManually<TValue = AnyT>(currentRawValue: TValue): void;
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyBackspace(event: IKeyboardEvent): void;
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyUp(event: IKeyboardEvent): void;
    /**
     * @stable [03.09.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyDown(event: IKeyboardEvent): void;
    /**
     * @stable [16.01.2020]
     * @param {IBaseEvent} event
     */
    onKeyEnter(event: IBaseEvent): void;
    /**
     * @stable [04.09.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyTab(event: IKeyboardEvent): void;
    /**
     * @stable [11.01.2020]
     * @param {IBaseEvent} event
     */
    onKeyEscape(event: IBaseEvent): void;
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyArrowDown(event: IKeyboardEvent): void;
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    onKeyArrowUp(event: IKeyboardEvent): void;
    /**
     * @stable [21.12.2019]
     * @returns {AnyT}
     */
    readonly value: AnyT;
    /**
     * @stable [17.06.2018]
     * @param {AnyT} event
     * @returns {AnyT}
     */
    abstract getRawValueFromEvent(event: AnyT): AnyT;
    /**
     * @stable [06.06.2018]
     */
    abstract setFocus(): void;
    /**
     * @stable [29.10.2019]
     * @param {boolean} usePrintf
     * @param {AnyT} args
     * @returns {string}
     */
    protected buildDisplayMessage(usePrintf: boolean, ...args: AnyT[]): string;
    /**
     * @stable [30.10.2019]
     * @returns {boolean}
     */
    protected readonly isValueNotPresent: boolean;
    /**
     * @stable [29.10.2019]
     * @returns {boolean}
     */
    protected readonly isRequired: boolean;
    /**
     * @stable [05.10.2018]
     * @returns {boolean}
     */
    protected isFieldInvalid(): boolean;
    /**
     * @stable [31.07.2018]
     * @param {AnyT} value
     * @returns {string}
     */
    protected validateValueAndSetCustomValidity(value: AnyT): string;
    /**
     * @stable [31.07.2018]
     * @param {AnyT} value
     * @returns {string}
     */
    protected validateValue(value: AnyT): string;
    /**
     * @stable [03.09.2018]
     * @returns {boolean}
     */
    protected isInputValid(): boolean;
    /**
     * @stable [31.07.2018]
     * @returns {string}
     */
    protected getNativeInputValidationMessage(): string;
    /**
     * @stable [31.07.2018]
     * @param {string} error
     */
    protected setNativeInputValidity(error: string): void;
    /**
     * @stable [17.06.2018]
     * @returns {Array<string | RegExp>}
     */
    protected getFieldMask(): Array<string | RegExp>;
    /**
     * @stable [17.06.2018]
     * @returns {string}
     */
    protected getFieldPattern(): string;
    /**
     * @stable [28.01.2020]
     * @param {AnyT} value
     * @param {boolean} forceApplyValue
     * @returns {AnyT}
     */
    protected getDecoratedDisplayValue(value: AnyT, forceApplyValue?: boolean): AnyT;
    /**
     * @stable [07.01.2018]
     * @param {AnyT} value
     * @returns {AnyT}
     */
    protected decorateDisplayValue(value: AnyT): AnyT;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected readonly isInactive: boolean;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected readonly isActive: boolean;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected readonly isDisabled: boolean;
    /**
     * @stable [29.10.2019]
     * @returns {boolean}
     */
    protected readonly isReadOnly: boolean;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected readonly isChangeable: boolean;
    /**
     * @stable [30.10.2019]
     * @returns {boolean}
     */
    protected readonly isFocused: boolean;
    /**
     * @stable [25.10.2019]
     * @returns {boolean}
     */
    protected readonly isVisible: boolean;
    /**
     * @stable [15.01.2020]
     * @returns {boolean}
     */
    protected isFieldBusy(): boolean;
    /**
     * @stable [30.10.2019]
     * @param {IFocusEvent} event
     */
    protected onFocus(event: IFocusEvent): void;
    /**
     * @stable [30.10.2019]
     * @param {IFocusEvent} event
     */
    protected onBlur(event: IFocusEvent): void;
    /**
     * @stable [22.01.2020]
     * @returns {JSX.Element}
     */
    protected readonly keyboardElement: JSX.Element;
    /**
     * @stable [22.01.2020]
     * @returns {JSX.Element}
     */
    protected readonly inputAttachmentElement: JSX.Element;
    /**
     * @stable [22.01.2020]
     * @returns {JSX.Element}
     */
    protected readonly actionsElement: JSX.Element;
    /**
     * @stable [22.01.2020]
     * @returns {JSX.Element}
     */
    protected readonly prefixLabelElement: JSX.Element;
    /**
     * @react-native-compatible
     * @stable [28.01.2020]
     * @returns {React.ReactNode}
     */
    protected getDisplayValueElement(): React.ReactNode;
    /**
     * @stable [23.12.2019]
     * @react-native-compatible
     * @returns {React.ReactNode}
     */
    protected getWaitMessageElement(): React.ReactNode;
    /**
     * @stable [28.10.2019]
     */
    protected openVirtualKeyboard(callback?: () => void): void;
    /**
     * @stable [28.10.2019]
     */
    protected closeVirtualKeyboard(): void;
    /**
     * @stable [28.10.2019]
     */
    protected onCloseVirtualKeyboard(): void;
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    protected isKeyboardOpen(): boolean;
    /**
     * @stable [24.10.2019]
     * @returns {boolean}
     */
    protected readonly isFocusPrevented: boolean;
    /**
     * @stable [24.10.2019]
     * @returns {boolean}
     */
    protected readonly isKeyboardUsed: boolean;
    /**
     * @stable [28.01.2020]
     * @returns {boolean}
     */
    protected readonly isPlainValueApplied: boolean;
    /**
     * @stable [28.01.2020]
     * @returns {boolean}
     */
    protected readonly isDisplayValueRenderedOnly: boolean;
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    protected getMessageElement(): JSX.Element;
    /**
     * @stable [03.09.2018]
     * @returns {string}
     */
    protected readonly error: string;
    /**
     * @stable [04.09.2018]
     */
    protected refreshCaretPosition(): void;
    /**
     * @stable [04.09.2018]
     * @returns {number}
     */
    protected getCaretPosition(): number;
    /**
     * @stable [30.10.2019]
     * @returns {boolean}
     */
    protected readonly isSyntheticCursorUsed: boolean;
    /**
     * @stable [30.01.2020]
     * @param {AnyT} value
     * @returns {boolean}
     */
    protected isValueObject(value: AnyT): boolean;
    /**
     * @stable [21.12.2019]
     * @param value
     * @returns {boolean}
     */
    protected isValueDefined(value: AnyT): boolean;
    /**
     * @stable [21.12.2019]
     * @returns {boolean}
     */
    protected readonly isValuePresent: boolean;
    /**
     * @stable [21.12.2019]
     * @returns {boolean}
     */
    protected readonly isDisplayValueDefined: boolean;
    /**
     * @stable [22.01.2020]
     * @returns {boolean}
     */
    protected readonly isFieldRendered: boolean;
    /**
     * @stable [21.12.2019]
     * @returns {AnyT}
     */
    protected readonly defaultValue: AnyT;
    /**
     * @stable [28.01.2020]
     * @returns {AnyT}
     */
    protected readonly displayValue: AnyT;
    /**
     * @stable [21.12.2019]
     * @returns {AnyT}
     */
    protected readonly decoratedDisplayValue: AnyT;
    /**
     * @stable [21.12.2019]
     * @returns {AnyT}
     */
    protected readonly originalEmptyValue: AnyT;
    /**
     * @stable [16.01.2020]
     * @returns {string}
     */
    protected getLabel(): string;
    /**
     * @stable [21.12.2019]
     * @returns {AnyT}
     */
    protected readonly emptyValue: AnyT;
    /**
     * @stable [03.09.2018]
     */
    protected abstract removeFocus(): void;
    /**
     * @stable [22.03.2019]
     * @param {string} message
     * @param {string} className
     * @returns {JSX.Element}
     */
    protected abstract toMessageElement(message: string, className?: string): JSX.Element;
    /**
     * @stable [31.07.2018]
     * @param {AnyT} currentRawValue
     */
    private onChangeValue;
    /**
     * @stable [31.07.2018]
     * @param {AnyT} rawValue
     */
    private validateField;
    /**
     * @stable [16.11.2019]
     * @param {AnyT} rawValue
     */
    private notifyFormAboutChanges;
    /**
     * @stable [16.11.2019]
     * @param {AnyT} rawValue
     */
    private notifyAboutChanges;
    /**
     * @stable [09.11.2019]
     */
    private setCaretVisibility;
}
