"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var util_1 = require("../../util");
var error_1 = require("../../error");
var definition_1 = require("../../definition");
var form_action_builder_1 = require("./form-action.builder");
/**
 * @stable [03.02.2020]
 * @param {IFieldChangeEntity & IFieldsChangesEntity} payload
 * @returns {IKeyValue}
 */
var fromPayload = function (payload) {
    var _a;
    var fieldChangeEntity = payload;
    var fieldsChangesEntity = payload;
    var fields = fieldsChangesEntity.fields;
    return Array.isArray(fields)
        ? R.mergeAll(fields.map(function (elem) {
            var _a;
            return (_a = {}, _a[elem.name] = elem.value, _a);
        }))
        : (_a = {}, _a[fieldChangeEntity.name] = fieldChangeEntity.value, _a);
};
function formReducer(state, action) {
    if (state === void 0) { state = definition_1.INITIAL_FORM_ENTITY; }
    var section = util_1.toSection(action);
    switch (action.type) {
        case form_action_builder_1.FormActionBuilder.buildDestroyActionType(section):
            return __assign({}, definition_1.INITIAL_FORM_ENTITY);
        case form_action_builder_1.FormActionBuilder.buildClearActionType(section):
        case form_action_builder_1.FormActionBuilder.buildChangeActionType(section):
            var changes = util_1.defValuesFilter(__assign({}, state.changes, fromPayload(action.data)));
            return __assign({}, state, { error: null, dirty: Object.keys(changes).length > 0, touched: true, changes: changes });
        case form_action_builder_1.FormActionBuilder.buildDefaultChangeActionType(section):
            var defaultChanges = util_1.defValuesFilter(__assign({}, state.defaultChanges, fromPayload(action.data)));
            return __assign({}, state, { defaultChanges: defaultChanges });
        case form_action_builder_1.FormActionBuilder.buildValidActionType(section):
            var formValidEntity = action.data;
            return __assign({}, state, { valid: formValidEntity.valid });
        case form_action_builder_1.FormActionBuilder.buildActiveValueActionType(section):
            /**
             * @stable [14.08.2018]
             */
            var activeValuePayload = action.data;
            return __assign({}, state, { activeValue: activeValuePayload.payload });
        case form_action_builder_1.FormActionBuilder.buildProgressActionType(section):
        case form_action_builder_1.FormActionBuilder.buildSubmitActionType(section):
            return __assign({}, state, { error: null, progress: true });
        case form_action_builder_1.FormActionBuilder.buildSubmitErrorActionType(section):
            return __assign({}, state, { progress: false, error: error_1.convertError(action.error).message });
        case form_action_builder_1.FormActionBuilder.buildSubmitFinishedActionType(section):
            return __assign({}, state, { progress: false });
        case form_action_builder_1.FormActionBuilder.buildResetActionType(section):
        case form_action_builder_1.FormActionBuilder.buildSubmitDoneActionType(section):
            /**
             * @stable [14.08.2018]
             */
            var activeValueState = { activeValue: state.activeValue };
            return __assign({}, definition_1.INITIAL_FORM_ENTITY, (util_1.isDef(activeValueState.activeValue) ? activeValueState : {}));
    }
    return state;
}
exports.formReducer = formReducer;
//# sourceMappingURL=form.reducer.js.map