"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var tabpanel_support_1 = require("../tabpanel/tabpanel.support");
var util_1 = require("../../util");
/**
 * @stable [29.05.2018]
 * @param {IFormProps} formProps
 * @param {IFieldProps} fieldProps
 * @returns {boolean}
 */
exports.isFormFieldDisabled = function (formProps, fieldProps) {
    return R.isNil(fieldProps.disabled)
        ? util_1.isFormDisabled(formProps)
        : fieldProps.disabled === true;
};
/**
 * @stable [16.11.2018]
 * @param {IFormProps} formProps
 * @param {IFieldProps} fieldProps
 * @returns {boolean}
 */
exports.isFormFieldChangeable = function (formProps, fieldProps) {
    return R.isNil(fieldProps.changeable)
        ? exports.isFormChangeable(formProps)
        : fieldProps.changeable !== false;
};
/**
 * @stable [16.11.2018]
 * @param {IFormProps} formProps
 * @returns {boolean}
 */
exports.isFormChangeable = function (formProps) { return formProps.changeable !== false; };
/**
 * @stable [15.02.2019]
 * @param {IFormProps} formProps
 * @returns {boolean}
 */
exports.isFormResettable = function (formProps) {
    return formProps.alwaysResettable === true
        || (util_1.isFormDirty(formProps) && !util_1.isFormDisabled(formProps));
};
/**
 * @stable [30.08.2018]
 * @param {IFormProps} formProps
 * @param {IGenericTabEntity} tab
 * @returns {boolean}
 */
exports.isFormTabActive = function (formProps, tab) {
    return tabpanel_support_1.isTabActive(formProps.form, tab);
};
//# sourceMappingURL=form.support.js.map