"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var util_1 = require("../../util");
var base_component_1 = require("../base/base.component");
var button_1 = require("../button");
var definition_1 = require("../../definition");
var menu_1 = require("../menu");
var link_1 = require("../link");
var Header = /** @class */ (function (_super) {
    __extends(Header, _super);
    /**
     * @stable [06.02.2020]
     * @param {IHeaderProps} props
     */
    function Header(props) {
        var _this = _super.call(this, props) || this;
        _this.defaultMenuProps = {
            options: [
                { label: _this.settings.messages.SETTINGS, icon: 'settings', value: definition_1.HeaderUserMenuActionsEnum.PROFILE },
                { label: _this.settings.messages.LOG_OUT, icon: 'sign_out_alt', value: definition_1.HeaderUserMenuActionsEnum.EXIT }
            ],
        };
        _this.defaultMenuActionProps = {
            icon: 'more',
        };
        _this.menuAnchorRef = React.createRef();
        _this.menuRef = React.createRef();
        _this.getMenuAnchorElement = _this.getMenuAnchorElement.bind(_this);
        _this.onMenuActionClick = _this.onMenuActionClick.bind(_this);
        _this.onMenuItemSelect = _this.onMenuItemSelect.bind(_this);
        return _this;
    }
    /**
     * @stable [06.02.2020]
     * @returns {JSX.Element}
     */
    Header.prototype.render = function () {
        var props = this.props;
        var user = props.user;
        var headerContentElement = this.headerContentElement;
        return (React.createElement("div", { className: util_1.joinClassName('rac-header', util_1.calc(props.className)) },
            headerContentElement && (React.createElement("div", { className: 'rac-header__content' }, headerContentElement)),
            React.createElement("div", { className: 'rac-header__actions' },
                props.children,
                React.createElement(link_1.Link, { to: this.routes.profile, className: 'rac-header__user-avatar', style: { backgroundImage: this.domAccessor.asImageUrl(user.url || this.settings.urls.emptyAvatar) } }),
                React.createElement(link_1.Link, { to: this.routes.profile, className: 'rac-header__user-info' }, user.name),
                this.menuActionElement)));
    };
    Object.defineProperty(Header.prototype, "headerContentElement", {
        /**
         * @stable [12.02.2020]
         * @returns {React.ReactNode}
         */
        get: function () {
            var _a = this.props, _b = _a.backActionConfiguration, backActionConfiguration = _b === void 0 ? {} : _b, content = _a.content;
            if (this.isBackActionRendered) {
                return (React.createElement(React.Fragment, null,
                    React.createElement(button_1.Button, __assign({ icon: 'back2' }, backActionConfiguration, { className: util_1.joinClassName('rac-header__back-action', util_1.calc(backActionConfiguration.className)) })),
                    content));
            }
            return content;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Header.prototype, "menuActionElement", {
        /**
         * @stable [06.02.2020]
         * @returns {JSX.Element}
         */
        get: function () {
            var props = this.props;
            var _a = props.menuActionConfiguration, menuActionConfiguration = _a === void 0 ? {} : _a, menuConfiguration = props.menuConfiguration;
            return (React.createElement(React.Fragment, null,
                React.createElement(button_1.Button, __assign({ ref: this.menuAnchorRef }, this.defaultMenuActionProps, menuActionConfiguration, { className: util_1.joinClassName('rac-header__menu-action', util_1.calc(menuActionConfiguration.className)), onClick: this.onMenuActionClick })),
                React.createElement(menu_1.Menu, __assign({ positionConfiguration: definition_1.DEFAULT_DOM_RIGHT_POSITION_CONFIG_ENTITY }, this.defaultMenuProps, menuConfiguration, { ref: this.menuRef, anchorElement: this.getMenuAnchorElement.bind(this), onSelect: this.onMenuItemSelect }))));
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [06.02.2020]
     * @param {IMenuItemEntity} menuItem
     */
    Header.prototype.onMenuItemSelect = function (menuItem) {
        var onSelect = this.props.onSelect;
        onSelect(menuItem);
    };
    /**
     * @stable [06.02.2020]
     */
    Header.prototype.onMenuActionClick = function () {
        this.menuRef.current.show();
    };
    /**
     * @stable [06.02.2020]
     * @returns {HTMLElement}
     */
    Header.prototype.getMenuAnchorElement = function () {
        return this.menuAnchorRef.current.getSelf();
    };
    Object.defineProperty(Header.prototype, "isBackActionRendered", {
        /**
         * @stable [12.02.2020]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isBackActionRendered(this.props);
        },
        enumerable: true,
        configurable: true
    });
    return Header;
}(base_component_1.BaseComponent));
exports.Header = Header;
//# sourceMappingURL=header.component.js.map